#coding utf-8

require_relative 'dataset.rb'
require_relative 'learner.rb'

module MachineLearner
  
  # ナイブベイズ学習器
  class BayesLearner < Learner
    
    # コンストラクタ
    def initialize
    end
    
    # データを元に学習を行う
    # @param datas [Array<DataSet>] トレーニングデータの配列
    # @return [Array<Boolean>] 識別結果の配列
    def learn(datas, ds = nil)
      # 学習データ
      @training = datas

      # 学習データがHashか
      @hash_mode = @training[0].x.kind_of?(Hash)

      # 学習データの結果のリスト
      @ys = @training.map{|data| data.y }
      @candidate_x = @hash_mode ? {} : []
    end
    
    # 識別を行う
    # @param x 特徴空間
    # @return [Fixnum] 識別結果
    def classify(xs)
      # 最も尤度の高い候補 y を探す
      classify_raw(xs).max{ |x, y| x[1] <=> y[1] }[0]
    end
    
    def classify_raw(xs)
      candidate_y.map{|y| [y, p_y_x(y, xs)]}.to_h
    end
    
    private
  
    # P(Y | X) を計算
    def p_y_x(y, xs)
      # P(Y | X) =  P(Y) * product(P(Xi | Y))
      p = Math.log(p_y(y))
      xs.each_with_index do |xi, i|
        i, xi = xi if @hash_mode
        next if xi.nil?
        p += Math.log(p_xi_y(xi, i, y))
      end
      return Math.exp(p)
    end
  
    # P(Xi | Y) を計算
    def p_xi_y(x, i, y)
      total = count = 0
      @ys.each_with_index do |ysj, j|
        next if ysj != y
        total += 1
        count += 1 if @training[j].x[i] == x
      end
      return (count + 1).to_f / (total + candidate_x(i).size + 1).to_f
    end
  
    # P(Y) を計算
    def p_y(y)
      count = @ys.count(y)
      return (count + 1).to_f / (@ys.size + candidate_y.size + 1).to_f
    end
  
    # xの候補を列挙
    def candidate_x(field)
      @candidate_x[field] ||= @training.map{|data| data.x[field]}.uniq.compact
    end
    
    # yの候補を列挙
    def candidate_y
      @candidate_y ||= @ys.uniq.compact
    end
  end
  
end

# vim: set et ts=2 sts=2 sw=2:

