#coding utf-8

module MachineLearner
  
  # 識別器クラス
  class Classifier
    
    # 識別を行う
    # @param x 特徴空間
    # @return [Fixnum] 識別結果
    def classify(x)
      raise NotImplementedError.new
    end
    
    # 性能の測定を行う
    # @param data[DataSet] データ
    # @return [Boolean] 識別できたらtrue, 外れたらfalse
    def test(data)
      classify(data.x) == data.y
    end
    
    # 性能の測定を行う
    # @param datas[Array<DataSet>] テストデータセット
    # @return [Float] 正解率(0以上1以下)
    def evaluate(datas)
      datas.count{|data| test(data)} / datas.size.to_f
    end
  end
  
  # 学習器クラス
  class Learner < Classifier
    
    # トレーニングデータを元に学習を行う
    # @param datas [Array<DataSet>] トレーニングデータセット
    # @param ds [Array<Float>] 各データセットの重み
    # @return [Array<Boolean>] 学習後の識別結果
    def learn(datas, ds)
      raise NotImplementedError.new
    end
  end
  
end

# vim: set et ts=2 sts=2 sw=2:
